package com.sf.vertx.handle;

import org.apache.commons.lang3.StringUtils;

import com.sf.vertx.constans.SacErrorCode;
import com.sf.vertx.utils.ProxyTool;

import io.vertx.core.http.HttpHeaders;
import io.vertx.core.json.JsonObject;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class RestfulFailureHandlerImpl implements RestfulFailureHandler {

	@Override
	public void handle(RoutingContext frc) {
		JsonObject errorJson = null;
		try {
			Throwable failure = frc.failure();
			if (failure instanceof HttpException) {
				HttpException httpException = (HttpException) failure;
				if (StringUtils.isNoneBlank(httpException.getPayload())) {
					errorJson = new JsonObject(httpException.getPayload());
				} else {
					errorJson = SacErrorCode.returnErrorMsg(httpException.getStatusCode());
				}
			} else {
				errorJson = SacErrorCode.returnErrorMsg(SacErrorCode.DEFAULT_ERROR_CODE);
			}
		} catch (Exception e) {
			e.printStackTrace();
			errorJson = SacErrorCode.returnErrorMsg(SacErrorCode.DEFAULT_ERROR_CODE);
		}
		
		frc.response().setChunked(true).setStatusCode(500).putHeader("Content-Type", "application/json")
				.putHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(errorJson.size())).end(errorJson.toBuffer());
		return;
	}

}
